	-- Author : N.Sauzede 2004,2005
	-- ADC : AD7822 
	--
	-- 8bit half-flash ADC with 420ns Conv Time
	-- 1 analog input channel + samples file

library ieee;
use ieee.std_logic_1164.all;		-- std_logic_vector
use ieee.std_logic_arith.all;		-- conv_std_logic
use ieee.std_logic_unsigned.all;	-- ditto
library std;
use std.textio.all;			-- to read/write files

entity adc is

	generic(	ND : integer := 8;	-- data width
			USE_FILE : integer := 1;	-- 1 to us a sample file ; 0 to sample input in1 (default)
			ADC_FILE : string := "adc.txt";	-- file containing adc samples
			NUMSAMPLES : integer := 400;	-- number of samples in file
			
			VERBOSE : integer := 0
		);

	port(	
		reset, convstb, csb, rdb : in bit;
		eocb : out bit;
		in1 : in integer;
		db : out std_logic_vector( (ND - 1) downto 0)
		);
end entity;

-- multi state trial for respect of rdb/eocb timing

architecture beh of adc is

	constant ADC_RESETT : integer := 1000;		-- real AD7822 takes up 1us with external ref ; 25us with internal one
--	constant ADC_RESETT : integer := 100;		-- real AD7822 takes up 1us with external ref ; 25us with internal one
	constant ADC_HOLDT : integer := 120;		-- real AD7822 takes up 120 ns to hold input to track
--	constant ADC_HOLDT : integer := 12;		-- real AD7822 takes up 120 ns to hold input to track
	constant ADC_CONVT : integer := 300;		-- real AD7822 takes up 300 ns to track conversion to tracked
--	constant ADC_CONVT : integer := 30;		-- real AD7822 takes up 300 ns to track conversion to tracked
	constant ADC_READT : integer := 70;		-- real AD7822 takes up 70 ns before returning from tracked to hold
--	constant ADC_READT : integer := 7;		-- real AD7822 takes up 70 ns before returning from tracked to hold
	constant CLKTIME : time := 0.5 ns;
	
	type adc_fsm is ( init, hold, track, tracked);
	signal state : adc_fsm;
	signal clk : bit;
	signal count : integer;
	type memory is array (integer range <>) of std_logic_vector( (ND-1) downto 0);
	signal samples : memory( 0 to (NUMSAMPLES-1));
	signal sample : integer;
	signal db_buf : std_logic_vector( (ND - 1) downto 0);
begin
	
	db <= (others => 'Z') when csb = '1' else db_buf;
--	db <= db_buf;
	
	pclk : process
	begin
		clk <= not clk;
		wait for CLKTIME;
	end process;
	
	main : process( reset, clk, convstb, rdb)
		variable s : line;
		variable n, num, gain : integer := 0;
		variable name : string(10 downto 1);
		file adc_file : text open read_mode is ADC_FILE;
	begin
		if (reset = '1' and reset'event) then
			eocb <= '0';
			state <= init;
			count <= ADC_RESETT - 1;
			sample <= 0;
			db_buf <= conv_std_logic_vector( 0, ND);
			if (USE_FILE = 1) then
				readline( adc_file, s);
				read( s, name);
				readline( adc_file, s);
				read( s, gain);
				readline( adc_file, s);
				read( s, n);
				readline( adc_file, s);
				if (VERBOSE = 1) then
					write( s, "About to load ");
					write( s, name);
					write( s, " : ");
					write( s, n);
					write( s, " samples");
					write( s, " gain=");
					write( s, gain);
					writeline( output, s);
				end if;
				for i in 0 to (n-1) loop
					readline( adc_file, s);
					read( s, num);
					samples( i) <= conv_std_logic_vector( num, ND);
					if (VERBOSE = 1) then
						write( s, "[adc] i=");
						write( s, i);
						write( s, " n=");
						write( s, num);
						writeline( output, s);
					end if;
				end loop;
			end if;
		elsif (clk'event and clk = '1') then
			case state is
				when init =>
					if (count = 0) then
						eocb <= '1';
						if (convstb = '0') then
							state <= hold;
							count <= ADC_HOLDT - 1;
						end if;
					else
						count <= count - 1;
					end if;
				when hold =>
					if (count = 0) then
						if (USE_FILE = 1) then
							db_buf <= samples( sample);
--							db_buf <= conv_std_logic_vector( samples(sample), ND);
							if (VERBOSE = 1) then
								write( s, "[adc] at ");
								write( s, now);
								write( s, ", sample=");
								write( s, sample);
								write( s, ", db_buf=");
								write( s, conv_integer( db_buf));
								writeline( output, s);
							end if;
							sample <= (sample + 1) mod NUMSAMPLES;
--							sample <= NUMSAMPLES;
						else
							db_buf <= conv_std_logic_vector( in1, ND);
						end if;
						state <= track;
						count <= ADC_CONVT - 1;
					else
						count <= count - 1;
					end if;
				when track =>
					if (count = 0) then
						eocb <= '0';
						state <= tracked;
						count <= ADC_READT - 1;
					else
						count <= count - 1;
					end if;
				when tracked =>
					if (rdb = '0') then
					end if;
					if (count = 0) then
						eocb <= '1';
						if (convstb = '0') then
							state <= hold;
							count <= ADC_HOLDT - 1;
						end if;
					else
						count <= count - 1;
					end if;
			end case;
		end if;
	end process;
end architecture;

-- good beh but doesn't respect rdb/eocb timing

--architecture beh of adc is
--
--	constant ADC_RESETT : integer := 1000;		-- real AD7822 takes up 1us with external ref ; 25us with internal one
----	constant ADC_RESETT : integer := 100;		-- real AD7822 takes up 1us with external ref ; 25us with internal one
--	constant ADC_HOLDT : integer := 120;		-- real AD7822 takes up 120 ns to hold input to track
----	constant ADC_HOLDT : integer := 12;		-- real AD7822 takes up 120 ns to hold input to track
--	constant ADC_CONVT : integer := 300;		-- real AD7822 takes up 300 ns to track conversion to tracked
----	constant ADC_CONVT : integer := 30;		-- real AD7822 takes up 300 ns to track conversion to tracked
--	constant ADC_READT : integer := 70;		-- real AD7822 takes up 70 ns before returning from tracked to hold
----	constant ADC_READT : integer := 7;		-- real AD7822 takes up 70 ns before returning from tracked to hold
--	constant CLKTIME : time := 1 ns;
--	
--	type adc_fsm is ( init, hold, track, tracked);
--	signal state : adc_fsm;
--	signal clk : bit;
--	signal count : integer;
--	type memory is array (integer range <>) of std_logic_vector( (ND-1) downto 0);
--	signal samples : memory( 0 to (NUMSAMPLES-1));
--	shared variable sample : integer := 0;
--	signal db_buf : std_logic_vector( (ND - 1) downto 0);
--begin
--	Zstate <= 	0 when (state = init) else
--			1 when (state = hold) else
--			2 when (state = track) else
--			3 when (state = tracked) else
--			4;
--	Zclk <= clk;
--	Zcount <= count;
--	
--	db <= (others => 'Z') when csb = '1' else db_buf;
----	db <= db_buf;
--	
--	pclk : process
--	begin
--		clk <= not clk;
--		wait for CLKTIME;
--	end process;
--	
--	main : process( reset, clk, convstb, rdb)
--		variable s : line;
--		variable num : integer := 0;
--		file adc_file : text open read_mode is ADC_FILE;
--	begin
--		if (reset'event and reset = '1') then
--			eocb <= '0';
--			state <= init;
--			count <= ADC_RESETT - 1;
--			sample:= 0;
--			db_buf <= conv_std_logic_vector( 0, ND);
--			if (USE_FILE = 1) then
--				for i in 0 to (NUMSAMPLES-1) loop
--					readline( adc_file, s);
--					read( s, num);
--					samples( i) <= conv_std_logic_vector( num, ND);
--					if (VERBOSE = 1) then
--						write( s, "[adc] i=");
--						write( s, i);
--						write( s, " n=");
--						write( s, num);
--						writeline( output, s);
--					end if;
--				end loop;
--			end if;
--		elsif (clk'event) then
--			case state is
--				when init =>
--					if (count = 0) then
--						eocb <= '1';
--						if (convstb = '0') then
--							state <= hold;
--							count <= ADC_HOLDT - 1;
--						end if;
--					else
--						count <= count - 1;
--					end if;
--				when track =>
--					if (count = 0) then
--						eocb <= '0';
--						state <= tracked;
--						count <= ADC_READT - 1;
--					else
--						count <= count - 1;
--					end if;
--				when tracked =>
--					if (rdb = '0') then
--						if (USE_FILE = 1) then
--							db_buf <= samples( sample);
--						else
--							db_buf <= conv_std_logic_vector( in1, ND);
--						end if;
--					end if;
--					if (count = 0) then
--						eocb <= '1';
--						if (convstb = '0') then
--							sample := (sample + 1) mod NUMSAMPLES;
--							state <= hold;
--							count <= ADC_HOLDT - 1;
--						end if;
--					else
--						count <= count - 1;
--					end if;
--				when hold =>
--					if (count = 0) then
--						state <= track;
--						count <= ADC_CONVT - 1;
--					else
--						count <= count - 1;
--					end if;
--			end case;
--		end if;
--	end process;
--end architecture;







