library ieee;
use ieee.std_logic_1164.all;		-- std_logic_vector
use ieee.std_logic_arith.all;		-- conv_std_logic
use ieee.std_logic_unsigned.all;	-- ditto

entity fir is
	generic(	NA : integer := 5;	-- address width 2**5=32
			ND : integer := 8;	-- data width
			NDO : integer := 21;	-- data out width
			NDB : integer := 21;	-- buffer data width
			ROM_FILE : string := "rom.txt");	-- file containing rom coefs

	port(	filter_in : in std_logic_vector( (ND - 1) downto 0);
		clk, reset, adc_busy : in bit;
		adc_convstb, adc_rd_csb : out bit;
		filter_out : out std_logic_vector( (NDO - 1) downto 0)
		);
end entity;

architecture beh of fir is

component fsm is
	generic(	NA : integer := NA);
	port(	clk, reset, adc_busy : in bit;
		adc_convstb, adc_rd_csb : out bit;
		rom_address : out std_logic_vector( (NA - 1) downto 0);
		delay_line_address : out std_logic_vector( (NA - 1) downto 0);
		delay_line_sample_shift, accu_ctrl, buff_oe : out bit
		);
end component;

component delay_line is
	generic(	NA : integer := NA;	-- address width
			ND : integer := ND;	-- data width
			
			VERBOSE : integer := 0
		);

	port(	delay_line_in : in std_logic_vector( (ND - 1) downto 0);
		delay_line_address : in std_logic_vector( (NA - 1) downto 0);
		delay_line_sample_shift, reset, clk : in bit;
		delay_line_out : out std_logic_vector( (ND - 1) downto 0)
		);
end component;

component buff is
	generic(	ND : integer := NDB	-- data width
		);

	port(	buff_in : in std_logic_vector( (NDB - 1) downto 0);
		buff_oe : in bit;
		clk : in bit;
		reset : in bit;
		buff_out : out std_logic_vector( (NDB - 1) downto 0));
end component;

component accu is
	generic(	NDI : integer := 2*ND;
			NDO : integer := NDO
		);
	port(	accu_in : in std_logic_vector( (2*ND - 1) downto 0);
		accu_ctrl : in bit;
		clk : in bit;
		reset : in bit;
		accu_out : out std_logic_vector( (NDO - 1) downto 0));
end component;

component rom is
	generic(	NA : integer := NA;	-- address width
			ND : integer := ND;	-- data width
			ROM_FILE : string := ROM_FILE);	-- file containing rom coefs

	port(	rom_address : in std_logic_vector( (NA-1) downto 0);
		reset, clk : in bit;
		rom_out : out std_logic_vector( (ND-1) downto 0)
		);

end component;

component mult is
	port(	mult_in_a : in std_logic_vector( (ND - 1) downto 0);
		mult_in_b : in std_logic_vector( (ND - 1) downto 0);
		mult_out : out std_logic_vector( (2*ND - 1) downto 0));
end component;

	signal rom_address : std_logic_vector( (NA - 1) downto 0);
	signal rom_out : std_logic_vector( (ND-1) downto 0);
	
	signal accu_ctrl, buff_oe, delay_line_sample_shift : bit;
	signal accu_out : std_logic_vector( (NDO - 1) downto 0);

	signal delay_line_address : std_logic_vector( (NA - 1) downto 0);

	signal delay_line_out : std_logic_vector( (ND - 1) downto 0);

	signal buff_in : std_logic_vector( (NDB - 1) downto 0);
	signal buff_out : std_logic_vector( (NDB - 1) downto 0);

	signal mult_out : std_logic_vector( (2*ND - 1) downto 0);

begin
	f1 : fsm
		port map( clk, reset, adc_busy, adc_convstb, adc_rd_csb, 
			rom_address, delay_line_address, delay_line_sample_shift, accu_ctrl, buff_oe
		);
	
	d1 : delay_line
		port map( filter_in, delay_line_address, delay_line_sample_shift, reset, clk, delay_line_out);
	
	b1 : buff
		port map( buff_in, buff_oe, clk, reset, filter_out);
	
	a1 : accu
		port map( mult_out, accu_ctrl, clk, reset, accu_out);
	
	r1 : rom
		port map( rom_address, reset, clk, rom_out);
	
	m1 : mult
		port map( delay_line_out, rom_out, mult_out);

	buff_in <= accu_out( (NDO - 1) downto (NDO - NDB));
	
end architecture;

